from .base_classes import *
from .support_functions import *


class MedicationAdministration(Resource):
    """Describes the event of a patient being given a dose of a medication.
    This may be as simple as swallowing a tablet or it may be a long
    running infusion. Related resources tie this event to the
    authorizing prescription, and the specific encounter between
    patient and health care practitioner.If the element is present,
    it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, status=None, patient=None, practitioner=None, encounter=None, prescription=None, wasNotGiven=None, reasonNotGiven=None, whenGiven=None, medication=None, device=None, dosage=None):
        self.original_tagname_ = None
        super(MedicationAdministration, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.status = status
        self.patient = patient
        self.practitioner = practitioner
        self.encounter = encounter
        self.prescription = prescription
        self.wasNotGiven = wasNotGiven
        if reasonNotGiven is None:
            self.reasonNotGiven = []
        else:
            self.reasonNotGiven = reasonNotGiven
        self.whenGiven = whenGiven
        self.medication = medication
        if device is None:
            self.device = []
        else:
            self.device = device
        if dosage is None:
            self.dosage = []
        else:
            self.dosage = dosage
    def factory(*args_, **kwargs_):
        if MedicationAdministration.subclass:
            return MedicationAdministration.subclass(*args_, **kwargs_)
        else:
            return MedicationAdministration(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_status(self): return self.status
    def set_status(self, status): self.status = status
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_practitioner(self): return self.practitioner
    def set_practitioner(self, practitioner): self.practitioner = practitioner
    def get_encounter(self): return self.encounter
    def set_encounter(self, encounter): self.encounter = encounter
    def get_prescription(self): return self.prescription
    def set_prescription(self, prescription): self.prescription = prescription
    def get_wasNotGiven(self): return self.wasNotGiven
    def set_wasNotGiven(self, wasNotGiven): self.wasNotGiven = wasNotGiven
    def get_reasonNotGiven(self): return self.reasonNotGiven
    def set_reasonNotGiven(self, reasonNotGiven): self.reasonNotGiven = reasonNotGiven
    def add_reasonNotGiven(self, value): self.reasonNotGiven.append(value)
    def insert_reasonNotGiven(self, index, value): self.reasonNotGiven[index] = value
    def get_whenGiven(self): return self.whenGiven
    def set_whenGiven(self, whenGiven): self.whenGiven = whenGiven
    def get_medication(self): return self.medication
    def set_medication(self, medication): self.medication = medication
    def get_device(self): return self.device
    def set_device(self, device): self.device = device
    def add_device(self, value): self.device.append(value)
    def insert_device(self, index, value): self.device[index] = value
    def get_dosage(self): return self.dosage
    def set_dosage(self, dosage): self.dosage = dosage
    def add_dosage(self, value): self.dosage.append(value)
    def insert_dosage(self, index, value): self.dosage[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.status is not None or
            self.patient is not None or
            self.practitioner is not None or
            self.encounter is not None or
            self.prescription is not None or
            self.wasNotGiven is not None or
            self.reasonNotGiven or
            self.whenGiven is not None or
            self.medication is not None or
            self.device or
            self.dosage or
            super(MedicationAdministration, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationAdministration', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministration')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationAdministration', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationAdministration'):
        super(MedicationAdministration, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministration')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationAdministration', fromsubclass_=False, pretty_print=True):
        super(MedicationAdministration, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.status is not None:
            self.status.export(outfile, level, namespace_, name_='status', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        if self.practitioner is not None:
            self.practitioner.export(outfile, level, namespace_, name_='practitioner', pretty_print=pretty_print)
        if self.encounter is not None:
            self.encounter.export(outfile, level, namespace_, name_='encounter', pretty_print=pretty_print)
        if self.prescription is not None:
            self.prescription.export(outfile, level, namespace_, name_='prescription', pretty_print=pretty_print)
        if self.wasNotGiven is not None:
            self.wasNotGiven.export(outfile, level, namespace_, name_='wasNotGiven', pretty_print=pretty_print)
        for reasonNotGiven_ in self.reasonNotGiven:
            reasonNotGiven_.export(outfile, level, namespace_, name_='reasonNotGiven', pretty_print=pretty_print)
        if self.whenGiven is not None:
            self.whenGiven.export(outfile, level, namespace_, name_='whenGiven', pretty_print=pretty_print)
        if self.medication is not None:
            self.medication.export(outfile, level, namespace_, name_='medication', pretty_print=pretty_print)
        for device_ in self.device:
            device_.export(outfile, level, namespace_, name_='device', pretty_print=pretty_print)
        for dosage_ in self.dosage:
            dosage_.export(outfile, level, namespace_, name_='dosage', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationAdministration', mapping_=None):
        element = super(MedicationAdministration, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.status is not None:
            status_ = self.status
            status_.to_etree(element, name_='status', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        if self.practitioner is not None:
            practitioner_ = self.practitioner
            practitioner_.to_etree(element, name_='practitioner', mapping_=mapping_)
        if self.encounter is not None:
            encounter_ = self.encounter
            encounter_.to_etree(element, name_='encounter', mapping_=mapping_)
        if self.prescription is not None:
            prescription_ = self.prescription
            prescription_.to_etree(element, name_='prescription', mapping_=mapping_)
        if self.wasNotGiven is not None:
            wasNotGiven_ = self.wasNotGiven
            wasNotGiven_.to_etree(element, name_='wasNotGiven', mapping_=mapping_)
        for reasonNotGiven_ in self.reasonNotGiven:
            reasonNotGiven_.to_etree(element, name_='reasonNotGiven', mapping_=mapping_)
        if self.whenGiven is not None:
            whenGiven_ = self.whenGiven
            whenGiven_.to_etree(element, name_='whenGiven', mapping_=mapping_)
        if self.medication is not None:
            medication_ = self.medication
            medication_.to_etree(element, name_='medication', mapping_=mapping_)
        for device_ in self.device:
            device_.to_etree(element, name_='device', mapping_=mapping_)
        for dosage_ in self.dosage:
            dosage_.to_etree(element, name_='dosage', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationAdministration, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'status':
            obj_ = MedicationAdministrationStatus.factory()
            obj_.build(child_)
            self.status = obj_
            obj_.original_tagname_ = 'status'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'practitioner':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.practitioner = obj_
            obj_.original_tagname_ = 'practitioner'
        elif nodeName_ == 'encounter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.encounter = obj_
            obj_.original_tagname_ = 'encounter'
        elif nodeName_ == 'prescription':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.prescription = obj_
            obj_.original_tagname_ = 'prescription'
        elif nodeName_ == 'wasNotGiven':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.wasNotGiven = obj_
            obj_.original_tagname_ = 'wasNotGiven'
        elif nodeName_ == 'reasonNotGiven':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reasonNotGiven.append(obj_)
            obj_.original_tagname_ = 'reasonNotGiven'
        elif nodeName_ == 'whenGiven':
            obj_ = Period.factory()
            obj_.build(child_)
            self.whenGiven = obj_
            obj_.original_tagname_ = 'whenGiven'
        elif nodeName_ == 'medication':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.medication = obj_
            obj_.original_tagname_ = 'medication'
        elif nodeName_ == 'device':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.device.append(obj_)
            obj_.original_tagname_ = 'device'
        elif nodeName_ == 'dosage':
            obj_ = MedicationAdministration_Dosage.factory()
            obj_.build(child_)
            self.dosage.append(obj_)
            obj_.original_tagname_ = 'dosage'
        super(MedicationAdministration, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationAdministration


class MedicationAdministration_Dosage(BackboneElement):
    """Describes the event of a patient being given a dose of a medication.
    This may be as simple as swallowing a tablet or it may be a long
    running infusion. Related resources tie this event to the
    authorizing prescription, and the specific encounter between
    patient and health care practitioner.The timing schedule for
    giving the medication to the patient. This may be a single time
    point (using dateTime) or it may be a start and end dateTime
    (Period).If set to true or if specified as a CodeableConcept,
    indicates that the medication is only taken when needed within
    the specified schedule rather than at every scheduled dose. If a
    CodeableConcept is present, it indicates the pre-condition for
    taking the Medication."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, timingDateTime=None, timingPeriod=None, asNeededBoolean=None, asNeededCodeableConcept=None, site=None, route=None, method=None, quantity=None, rate=None, maxDosePerPeriod=None):
        self.original_tagname_ = None
        super(MedicationAdministration_Dosage, self).__init__(id, extension, modifierExtension, )
        self.timingDateTime = timingDateTime
        self.timingPeriod = timingPeriod
        self.asNeededBoolean = asNeededBoolean
        self.asNeededCodeableConcept = asNeededCodeableConcept
        self.site = site
        self.route = route
        self.method = method
        self.quantity = quantity
        self.rate = rate
        self.maxDosePerPeriod = maxDosePerPeriod
    def factory(*args_, **kwargs_):
        if MedicationAdministration_Dosage.subclass:
            return MedicationAdministration_Dosage.subclass(*args_, **kwargs_)
        else:
            return MedicationAdministration_Dosage(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_timingDateTime(self): return self.timingDateTime
    def set_timingDateTime(self, timingDateTime): self.timingDateTime = timingDateTime
    def get_timingPeriod(self): return self.timingPeriod
    def set_timingPeriod(self, timingPeriod): self.timingPeriod = timingPeriod
    def get_asNeededBoolean(self): return self.asNeededBoolean
    def set_asNeededBoolean(self, asNeededBoolean): self.asNeededBoolean = asNeededBoolean
    def get_asNeededCodeableConcept(self): return self.asNeededCodeableConcept
    def set_asNeededCodeableConcept(self, asNeededCodeableConcept): self.asNeededCodeableConcept = asNeededCodeableConcept
    def get_site(self): return self.site
    def set_site(self, site): self.site = site
    def get_route(self): return self.route
    def set_route(self, route): self.route = route
    def get_method(self): return self.method
    def set_method(self, method): self.method = method
    def get_quantity(self): return self.quantity
    def set_quantity(self, quantity): self.quantity = quantity
    def get_rate(self): return self.rate
    def set_rate(self, rate): self.rate = rate
    def get_maxDosePerPeriod(self): return self.maxDosePerPeriod
    def set_maxDosePerPeriod(self, maxDosePerPeriod): self.maxDosePerPeriod = maxDosePerPeriod
    def hasContent_(self):
        if (
            self.timingDateTime is not None or
            self.timingPeriod is not None or
            self.asNeededBoolean is not None or
            self.asNeededCodeableConcept is not None or
            self.site is not None or
            self.route is not None or
            self.method is not None or
            self.quantity is not None or
            self.rate is not None or
            self.maxDosePerPeriod is not None or
            super(MedicationAdministration_Dosage, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationAdministration.Dosage', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministration.Dosage')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationAdministration.Dosage', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationAdministration.Dosage'):
        super(MedicationAdministration_Dosage, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministration.Dosage')
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationAdministration.Dosage', fromsubclass_=False, pretty_print=True):
        super(MedicationAdministration_Dosage, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.timingDateTime is not None:
            self.timingDateTime.export(outfile, level, namespace_, name_='timingDateTime', pretty_print=pretty_print)
        if self.timingPeriod is not None:
            self.timingPeriod.export(outfile, level, namespace_, name_='timingPeriod', pretty_print=pretty_print)
        if self.asNeededBoolean is not None:
            self.asNeededBoolean.export(outfile, level, namespace_, name_='asNeededBoolean', pretty_print=pretty_print)
        if self.asNeededCodeableConcept is not None:
            self.asNeededCodeableConcept.export(outfile, level, namespace_, name_='asNeededCodeableConcept', pretty_print=pretty_print)
        if self.site is not None:
            self.site.export(outfile, level, namespace_, name_='site', pretty_print=pretty_print)
        if self.route is not None:
            self.route.export(outfile, level, namespace_, name_='route', pretty_print=pretty_print)
        if self.method is not None:
            self.method.export(outfile, level, namespace_, name_='method', pretty_print=pretty_print)
        if self.quantity is not None:
            self.quantity.export(outfile, level, namespace_, name_='quantity', pretty_print=pretty_print)
        if self.rate is not None:
            self.rate.export(outfile, level, namespace_, name_='rate', pretty_print=pretty_print)
        if self.maxDosePerPeriod is not None:
            self.maxDosePerPeriod.export(outfile, level, namespace_, name_='maxDosePerPeriod', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationAdministration.Dosage', mapping_=None):
        element = super(MedicationAdministration_Dosage, self).to_etree(parent_element, name_, mapping_)
        if self.timingDateTime is not None:
            timingDateTime_ = self.timingDateTime
            timingDateTime_.to_etree(element, name_='timingDateTime', mapping_=mapping_)
        if self.timingPeriod is not None:
            timingPeriod_ = self.timingPeriod
            timingPeriod_.to_etree(element, name_='timingPeriod', mapping_=mapping_)
        if self.asNeededBoolean is not None:
            asNeededBoolean_ = self.asNeededBoolean
            asNeededBoolean_.to_etree(element, name_='asNeededBoolean', mapping_=mapping_)
        if self.asNeededCodeableConcept is not None:
            asNeededCodeableConcept_ = self.asNeededCodeableConcept
            asNeededCodeableConcept_.to_etree(element, name_='asNeededCodeableConcept', mapping_=mapping_)
        if self.site is not None:
            site_ = self.site
            site_.to_etree(element, name_='site', mapping_=mapping_)
        if self.route is not None:
            route_ = self.route
            route_.to_etree(element, name_='route', mapping_=mapping_)
        if self.method is not None:
            method_ = self.method
            method_.to_etree(element, name_='method', mapping_=mapping_)
        if self.quantity is not None:
            quantity_ = self.quantity
            quantity_.to_etree(element, name_='quantity', mapping_=mapping_)
        if self.rate is not None:
            rate_ = self.rate
            rate_.to_etree(element, name_='rate', mapping_=mapping_)
        if self.maxDosePerPeriod is not None:
            maxDosePerPeriod_ = self.maxDosePerPeriod
            maxDosePerPeriod_.to_etree(element, name_='maxDosePerPeriod', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MedicationAdministration_Dosage, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'timingDateTime':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.timingDateTime = obj_
            obj_.original_tagname_ = 'timingDateTime'
        elif nodeName_ == 'timingPeriod':
            obj_ = Period.factory()
            obj_.build(child_)
            self.timingPeriod = obj_
            obj_.original_tagname_ = 'timingPeriod'
        elif nodeName_ == 'asNeededBoolean':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.asNeededBoolean = obj_
            obj_.original_tagname_ = 'asNeededBoolean'
        elif nodeName_ == 'asNeededCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.asNeededCodeableConcept = obj_
            obj_.original_tagname_ = 'asNeededCodeableConcept'
        elif nodeName_ == 'site':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.site = obj_
            obj_.original_tagname_ = 'site'
        elif nodeName_ == 'route':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.route = obj_
            obj_.original_tagname_ = 'route'
        elif nodeName_ == 'method':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.method = obj_
            obj_.original_tagname_ = 'method'
        elif nodeName_ == 'quantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.quantity = obj_
            obj_.original_tagname_ = 'quantity'
        elif nodeName_ == 'rate':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.rate = obj_
            obj_.original_tagname_ = 'rate'
        elif nodeName_ == 'maxDosePerPeriod':
            obj_ = Ratio.factory()
            obj_.build(child_)
            self.maxDosePerPeriod = obj_
            obj_.original_tagname_ = 'maxDosePerPeriod'
        super(MedicationAdministration_Dosage, self).buildChildren(child_, node, nodeName_, True)
# end class MedicationAdministration_Dosage


class MedicationAdministrationStatus(Element):
    """A set of codes indicating the current status of a
    MedicationAdministrationIf the element is present, it must have
    either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(MedicationAdministrationStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if MedicationAdministrationStatus.subclass:
            return MedicationAdministrationStatus.subclass(*args_, **kwargs_)
        else:
            return MedicationAdministrationStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(MedicationAdministrationStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MedicationAdministrationStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministrationStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MedicationAdministrationStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MedicationAdministrationStatus'):
        super(MedicationAdministrationStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MedicationAdministrationStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='MedicationAdministrationStatus', fromsubclass_=False, pretty_print=True):
        super(MedicationAdministrationStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MedicationAdministrationStatus', mapping_=None):
        element = super(MedicationAdministrationStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(MedicationAdministrationStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(MedicationAdministrationStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class MedicationAdministrationStatus



from .base_classes import *
from .support_functions import *


class OrderResponse(Resource):
    """A response to an order.If the element is present, it must have
    either a @value, an @id, or extensionsA reference to an
    authority policy that is the reason for the response. Usually
    this is used when the order is rejected, to provide a reason for
    rejection."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, request=None, date=None, who=None, authorityCodeableConcept=None, authorityResource=None, code=None, description=None, fulfillment=None):
        self.original_tagname_ = None
        super(OrderResponse, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.request = request
        self.date = date
        self.who = who
        self.authorityCodeableConcept = authorityCodeableConcept
        self.authorityResource = authorityResource
        self.code = code
        self.description = description
        if fulfillment is None:
            self.fulfillment = []
        else:
            self.fulfillment = fulfillment
    def factory(*args_, **kwargs_):
        if OrderResponse.subclass:
            return OrderResponse.subclass(*args_, **kwargs_)
        else:
            return OrderResponse(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_request(self): return self.request
    def set_request(self, request): self.request = request
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_who(self): return self.who
    def set_who(self, who): self.who = who
    def get_authorityCodeableConcept(self): return self.authorityCodeableConcept
    def set_authorityCodeableConcept(self, authorityCodeableConcept): self.authorityCodeableConcept = authorityCodeableConcept
    def get_authorityResource(self): return self.authorityResource
    def set_authorityResource(self, authorityResource): self.authorityResource = authorityResource
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_fulfillment(self): return self.fulfillment
    def set_fulfillment(self, fulfillment): self.fulfillment = fulfillment
    def add_fulfillment(self, value): self.fulfillment.append(value)
    def insert_fulfillment(self, index, value): self.fulfillment[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.request is not None or
            self.date is not None or
            self.who is not None or
            self.authorityCodeableConcept is not None or
            self.authorityResource is not None or
            self.code is not None or
            self.description is not None or
            self.fulfillment or
            super(OrderResponse, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='OrderResponse', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='OrderResponse')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='OrderResponse', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='OrderResponse'):
        super(OrderResponse, self).exportAttributes(outfile, level, already_processed, namespace_, name_='OrderResponse')
    def exportChildren(self, outfile, level, namespace_='', name_='OrderResponse', fromsubclass_=False, pretty_print=True):
        super(OrderResponse, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.request is not None:
            self.request.export(outfile, level, namespace_, name_='request', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.who is not None:
            self.who.export(outfile, level, namespace_, name_='who', pretty_print=pretty_print)
        if self.authorityCodeableConcept is not None:
            self.authorityCodeableConcept.export(outfile, level, namespace_, name_='authorityCodeableConcept', pretty_print=pretty_print)
        if self.authorityResource is not None:
            self.authorityResource.export(outfile, level, namespace_, name_='authorityResource', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        for fulfillment_ in self.fulfillment:
            fulfillment_.export(outfile, level, namespace_, name_='fulfillment', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='OrderResponse', mapping_=None):
        element = super(OrderResponse, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.request is not None:
            request_ = self.request
            request_.to_etree(element, name_='request', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.who is not None:
            who_ = self.who
            who_.to_etree(element, name_='who', mapping_=mapping_)
        if self.authorityCodeableConcept is not None:
            authorityCodeableConcept_ = self.authorityCodeableConcept
            authorityCodeableConcept_.to_etree(element, name_='authorityCodeableConcept', mapping_=mapping_)
        if self.authorityResource is not None:
            authorityResource_ = self.authorityResource
            authorityResource_.to_etree(element, name_='authorityResource', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        for fulfillment_ in self.fulfillment:
            fulfillment_.to_etree(element, name_='fulfillment', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(OrderResponse, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'request':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.request = obj_
            obj_.original_tagname_ = 'request'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'who':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.who = obj_
            obj_.original_tagname_ = 'who'
        elif nodeName_ == 'authorityCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.authorityCodeableConcept = obj_
            obj_.original_tagname_ = 'authorityCodeableConcept'
        elif nodeName_ == 'authorityResource':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authorityResource = obj_
            obj_.original_tagname_ = 'authorityResource'
        elif nodeName_ == 'code':
            obj_ = OrderOutcomeStatus.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'fulfillment':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.fulfillment.append(obj_)
            obj_.original_tagname_ = 'fulfillment'
        super(OrderResponse, self).buildChildren(child_, node, nodeName_, True)
# end class OrderResponse


class OrderOutcomeStatus(Element):
    """The status of the response to an orderIf the element is present, it
    must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(OrderOutcomeStatus, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if OrderOutcomeStatus.subclass:
            return OrderOutcomeStatus.subclass(*args_, **kwargs_)
        else:
            return OrderOutcomeStatus(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(OrderOutcomeStatus, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='OrderOutcomeStatus', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='OrderOutcomeStatus')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='OrderOutcomeStatus', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='OrderOutcomeStatus'):
        super(OrderOutcomeStatus, self).exportAttributes(outfile, level, already_processed, namespace_, name_='OrderOutcomeStatus')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='OrderOutcomeStatus', fromsubclass_=False, pretty_print=True):
        super(OrderOutcomeStatus, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='OrderOutcomeStatus', mapping_=None):
        element = super(OrderOutcomeStatus, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(OrderOutcomeStatus, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(OrderOutcomeStatus, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class OrderOutcomeStatus



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#                             Bruno M. Villasanti <bvillasanti@thymbra.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import optparse
import psycopg2


def main(pg_cursor):
    pg_cursor.execute('UPDATE "ir_model_data" '
        'SET inherit = %s '
        'WHERE fs_id like \'prod_em%%\' '
        '  AND model = %s AND module = %s ',
        (False, 'product.template', 'health',))

if __name__ == '__main__':
    parser = optparse.OptionParser(version='0.1')
    parser.add_option('--pg', dest='dsn', help='dsn for PostgreSQL')

    opt, args = parser.parse_args()
    pg_conn = None
    pg_cursor = None
    try:
        print ">> Attempting to connect..."
        pg_conn = psycopg2.connect(opt.dsn)
        print ">> Connected!"
        pg_cursor = pg_conn.cursor()
        main(pg_cursor)
        pg_conn.commit()
        print ">> Successfully prepared for migration!"
    finally:
        if pg_cursor:
            pg_cursor.close()
        if pg_conn:
            pg_conn.close()



from .base_classes import *
from .support_functions import *


class RelatedPerson(Resource):
    """Information about a person that is involved in the care for a
    patient, but who is not the target of healthcare, nor has a
    formal responsibility in the care process.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, patient=None, relationship=None, name=None, telecom=None, gender=None, address=None, photo=None):
        self.original_tagname_ = None
        super(RelatedPerson, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.patient = patient
        self.relationship = relationship
        self.name = name
        if telecom is None:
            self.telecom = []
        else:
            self.telecom = telecom
        self.gender = gender
        self.address = address
        if photo is None:
            self.photo = []
        else:
            self.photo = photo
    def factory(*args_, **kwargs_):
        if RelatedPerson.subclass:
            return RelatedPerson.subclass(*args_, **kwargs_)
        else:
            return RelatedPerson(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_patient(self): return self.patient
    def set_patient(self, patient): self.patient = patient
    def get_relationship(self): return self.relationship
    def set_relationship(self, relationship): self.relationship = relationship
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_telecom(self): return self.telecom
    def set_telecom(self, telecom): self.telecom = telecom
    def add_telecom(self, value): self.telecom.append(value)
    def insert_telecom(self, index, value): self.telecom[index] = value
    def get_gender(self): return self.gender
    def set_gender(self, gender): self.gender = gender
    def get_address(self): return self.address
    def set_address(self, address): self.address = address
    def get_photo(self): return self.photo
    def set_photo(self, photo): self.photo = photo
    def add_photo(self, value): self.photo.append(value)
    def insert_photo(self, index, value): self.photo[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.patient is not None or
            self.relationship is not None or
            self.name is not None or
            self.telecom or
            self.gender is not None or
            self.address is not None or
            self.photo or
            super(RelatedPerson, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='RelatedPerson', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='RelatedPerson')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='RelatedPerson', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='RelatedPerson'):
        super(RelatedPerson, self).exportAttributes(outfile, level, already_processed, namespace_, name_='RelatedPerson')
    def exportChildren(self, outfile, level, namespace_='', name_='RelatedPerson', fromsubclass_=False, pretty_print=True):
        super(RelatedPerson, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.patient is not None:
            self.patient.export(outfile, level, namespace_, name_='patient', pretty_print=pretty_print)
        if self.relationship is not None:
            self.relationship.export(outfile, level, namespace_, name_='relationship', pretty_print=pretty_print)
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        for telecom_ in self.telecom:
            telecom_.export(outfile, level, namespace_, name_='telecom', pretty_print=pretty_print)
        if self.gender is not None:
            self.gender.export(outfile, level, namespace_, name_='gender', pretty_print=pretty_print)
        if self.address is not None:
            self.address.export(outfile, level, namespace_, name_='address', pretty_print=pretty_print)
        for photo_ in self.photo:
            photo_.export(outfile, level, namespace_, name_='photo', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='RelatedPerson', mapping_=None):
        element = super(RelatedPerson, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.patient is not None:
            patient_ = self.patient
            patient_.to_etree(element, name_='patient', mapping_=mapping_)
        if self.relationship is not None:
            relationship_ = self.relationship
            relationship_.to_etree(element, name_='relationship', mapping_=mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        for telecom_ in self.telecom:
            telecom_.to_etree(element, name_='telecom', mapping_=mapping_)
        if self.gender is not None:
            gender_ = self.gender
            gender_.to_etree(element, name_='gender', mapping_=mapping_)
        if self.address is not None:
            address_ = self.address
            address_.to_etree(element, name_='address', mapping_=mapping_)
        for photo_ in self.photo:
            photo_.to_etree(element, name_='photo', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(RelatedPerson, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'patient':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.patient = obj_
            obj_.original_tagname_ = 'patient'
        elif nodeName_ == 'relationship':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.relationship = obj_
            obj_.original_tagname_ = 'relationship'
        elif nodeName_ == 'name':
            obj_ = HumanName.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'telecom':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.telecom.append(obj_)
            obj_.original_tagname_ = 'telecom'
        elif nodeName_ == 'gender':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.gender = obj_
            obj_.original_tagname_ = 'gender'
        elif nodeName_ == 'address':
            obj_ = Address.factory()
            obj_.build(child_)
            self.address = obj_
            obj_.original_tagname_ = 'address'
        elif nodeName_ == 'photo':
            obj_ = Attachment.factory()
            obj_.build(child_)
            self.photo.append(obj_)
            obj_.original_tagname_ = 'photo'
        super(RelatedPerson, self).buildChildren(child_, node, nodeName_, True)
# end class RelatedPerson



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
#
#    HL7 FHIR Python Reference Implementation
#
#    Copyright (C) 2014-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2014-2017 GNU Solidario <health@gnusolidario.org>
#    Copyright (C) 2014-2015 Chris Zimmerman <siv@riseup.net> 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import requests
import json
import lxml.etree


__all__ = ["RestfulFHIR"]


class RestfulFHIR:
    """General Set of REST Interactions for resources"""

    def __init__(self, base, content_type='json'):
        self.mimes={'xml': {'resource': 'application/xml+fhir; charset=utf-8',
                            'bundle': 'application/atom+xml; charset=utf-8',
                            'taglist': 'application/xml+fhir; charset=utf-8'},
                    'json': {'resource': 'application/json+fhir; charset=utf-8',
                            'bundle': 'application/json+fhir; charset=utf-8',
                            'taglist': 'application/json+fhir; charset=utf-8'}}
        self.validators={'json': json.loads,
                            'xml': lxml.etree.fromstring} 
        self.content_headers=self.mimes[content_type]
        self.validator=self.validators[content_type]
        self.base = base

    def __url_join(self, args):
        """Simple utility function to generate url from arguments
        """
        return '/'.join([str(arg) for arg in args if arg is not None])

    def search(self, resource, params):
        """Search FHIR Resources with specific criteria
            PARAMETERS:
                resource : resource type
                params : extra search criteria
            RETURNS:
                response
                    200 : Found
                    403 : Failed
        """

        fhir_query = self.__url_join([self.base, resource])
        headers={'accept': self.content_headers['resource']}
        response = requests.get(fhir_query, headers=headers, params=params)
        return response

    def read(self, resource, resid):
        """Read current status of the resource
            PARAMETERS:
                resource : resource type
                resid : unique resource identifier
            RETURNS:
                response
                    200 : Found
                    404 : Does not exist
                    410 : Deleted resource
        """

        fhir_query = self.__url_join([self.base, resource, resid])
        headers={'accept': self.content_headers['resource']}
        response = requests.get(fhir_query, headers=headers)
        return response

    def vread(self, resource, resid, vid):
        """Read given version of the resource
            PARAMETERS:
                resource : resource type
                resid : unique resource identifier
                vid : unique version identifier
            RETURNS:
                response
                    200 : Found
                    404 : Does not exist
                    405 : Prohibit previous versions
                    410 : Deleted resource
        """

        fhir_query = self.__url_join([self.base, resource, resid, '_history', vid])
        headers={'accept': self.content_headers['resource']}
        response = requests.get(fhir_query, headers=headers)
        return response

    def update(self, resource, resid, body):
        """Update or create FHIR resource
            PARAMETERS:
                resource : resource type
                resid : unique resource identifier
                body : resource
            RETURNS:
                response
                    200 : Resource updated
                    201 : Resource created
                    400 : Bad request
                    404 : Type not supported
                    405 : Not allowed
                    409 : Version conflict
                    412 : Version precondition failure
                    422 : Rejected
        """

        try:
            self.validator(body)
        except:
            raise TypeError("Body does not have a valid structure")
        fhir_query = self.__url_join([self.base, resource, resid])
        headers={'content-type': self.content_headers['resource']}
        response = requests.put(fhir_query, data=body, headers=headers)
        return response

    def delete(self, resource, resid):
        """Delete existing resource
            PARAMETERS:
                resource : resource type
                resid : unique resource identifier
            RETURNS:
                response
                    204 : Delete successful
                    404 : Does not exist
                    405 : Not allowed
        """

        fhir_query = self.__url_join([self.base, resource, resid])
        response = requests.delete(fhir_query)
        return response

    def create(self, resource, body):
        """Create new resource
            PARAMETERS:
                resource : resource type
                body : resource
            RETURNS:
                response
                    201 : Resource created
                    400 : Bad request
                    404 : Not supported
                    422 : Rejected
                    500 : Incorrect Document
        """

        try:
            self.validator(body)
        except:
            raise TypeError("Body does not have a valid structure")
        fhir_query = self.__url_join([self.base, resource])
        headers = {'content-type': self.content_headers['resource']}
        response = requests.post(fhir_query, data=body, headers=headers)
        return response

    def transaction(self, bundle):
        """Create, delete, or update multiple resources
            PARAMETERS:
                bundle : resource bundle
            RETURNS:
                response
                    200 : Success
                    400 : Bad request
                    404 : Not supported
                    405 : Not allowed
                    409 : Version conflict
                    412 : Version precondition conflict
                    422 : Rejected
        """

        try:
            self.validator(bundle)
        except:
            raise TypeError("Bundle does not have a valid structure")
        fhir_query = str(self.base)
        headers = {'content-type': self.content_headers['bundle']}
        response = requests.post(fhir_query, data=bundle, headers=headers)
        return response

    def conformance(self, _options=False):
        """Retrieves conformance statement
            PARAMETERS:
                _options : use OPTIONS verb
            RETURNS:
                response
                    200 : Found
                    404 : FHIR not supported
        """

        headers={'accept': self.content_headers['resource']}
        if _options:
            response = requests.options(str(self.base), headers=headers)
        else:
            fhir_query = self.__url_join([self.base, 'metadata'])
            response = requests.get(fhir_query, headers=headers)
        return response

    def history(self, resource=None, resid=None, params=None):
        """Retrieve history of: specific resource, given type, or all resources
            PARAMETERS:
                resource : resource type
                resid :  unique resource identifier
                params : extra history criteria
            RETURNS:
                response
                    200 : Found
        """
        fhir_query = self.__url_join([self.base, resource, resid, '_history'])
        headers={'accept': self.content_headers['resource']}
        response = requests.get(fhir_query, headers=headers, params=params)
        return response

    def validate(self, resource, resid=None, body=None):
        """Check whether content is a valid resource and update
            PARAMETERS:
                resource : resource type
                body : resource
                resid : unique resource identifier
            RETURNS:
                response
                    200 : Valid resource/Valid update
                    400 : Bad request/Invalid
                    422 : Valid resource, but invalid update
        """

        if not body: raise TypeError('Need something to validate! Attach body.')
        try:
            self.validator(body)
        except:
            raise TypeError("Body does not have a valid structure")
        fhir_query = self.__url_join([self.base, resource, '_validate', resid])
        headers = {'content-type': self.content_headers['resource']} 
        response = requests.post(fhir_query, data=body, headers=headers)
        return response



from flask import request
from server.common import get_userid, tryton
from server.common import Resource
from server.resources import SearchRecords
from server.resources import ReadRecord
from server.resources import ValidateRecord
from server.resources import Routing

class CO_Create(Resource):
    def post(self):
        '''Create interaction'''
        return 'not implemented', 405

class CO_Search(Resource):
    def get(self):
        action = SearchRecords(endpoint='condition', request=request)
        return action.records

class CO_Validate(Resource):
    def post(self, log_id=None):
        action = ValidateRecord(endpoint='condition', record=request.data)
        return action.valid

class CO_Record(Resource):
    def get(self, log_id):
        action = ReadRecord(endpoint='condition', log_id=log_id)
        return action.record

    def put(self, log_id):
        '''Update interaction'''
        return 'Not supported', 405

    def delete(self, log_id):
        '''Delete interaction'''
        return 'Not implemented', 405

class CO_Version(Resource):
    def get(self, log_id, v_id=None):
        '''Vread interaction'''
        return 'Not supported', 405


routing = Routing('Condition')
routing['create'] = CO_Create
routing['search'] = CO_Search
routing['validate'] = CO_Validate
routing['record'] = CO_Record
routing['version'] = CO_Version

__all__=['routing']



class administrativeGender:
    contents=[{'code': 'F',
            'display': 'Female',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'},
        {'code': 'M',
            'display': 'Male',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'},
        {'code': 'UN',
            'display': 'Undifferentiated',
            'system': 'http://hl7.org/fhir/v3/AdministrativeGender'}]

__all__=['administrativeGender']



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from dateutil.relativedelta import relativedelta
from datetime import datetime, timedelta, date
from trytond.model import ModelView, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal, And, Or, If
from trytond.pool import Pool

from trytond import backend

__all__ = ['Party','PatientSESAssessment','GnuHealthPatient']


class Party (ModelSQL, ModelView):
    __name__ = 'party.party'

    occupation = fields.Many2One('gnuhealth.occupation', 'Occupation')

    education = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
        ], 'Education', help="Education Level", sort=False)


class PatientSESAssessment(ModelSQL, ModelView):
    'Socioeconomics and Family Functionality Assessment'
    __name__ = 'gnuhealth.ses.assessment'

    STATES = {'readonly': Eval('state') == 'done'}

    patient = fields.Many2One('gnuhealth.patient', 'Patient',
        required=True, states = STATES)
    gender = fields.Function(fields.Selection([
        (None, ''),
        ('m', 'Male'),
        ('f', 'Female'),
        ], 'Gender'), 'get_patient_gender', searcher='search_patient_gender')


    assessment_date = fields.DateTime('Date', help="Assessment date",
        states = STATES)
        
    computed_age = fields.Function(fields.Char(
            'Age',
            help="Computed patient age at the moment of the evaluation"),
            'patient_age_at_assessment')
            
    health_professional = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Professional', readonly=True,
        help="Health professional"
        )

    du = fields.Many2One(
        'gnuhealth.du', 'DU', help="Domiciliary Unit",
        states = STATES)

    ses = fields.Selection([
        (None, ''),
        ('0', 'Lower'),
        ('1', 'Lower-middle'),
        ('2', 'Middle'),
        ('3', 'Middle-upper'),
        ('4', 'Higher'),
        ], 'Socioeconomics', help="SES - Socioeconomic Status", sort=False,
            states = STATES)

    housing = fields.Selection([
        (None, ''),
        ('0', 'Shanty, deficient sanitary conditions'),
        ('1', 'Small, crowded but with good sanitary conditions'),
        ('2', 'Comfortable and good sanitary conditions'),
        ('3', 'Roomy and excellent sanitary conditions'),
        ('4', 'Luxury and excellent sanitary conditions'),
        ], 'Housing conditions',
         help="Housing and sanitary living conditions", sort=False,
            states = STATES)

    occupation = fields.Many2One('gnuhealth.occupation','Occupation',
        states = STATES)

    income = fields.Selection([
        (None, ''),
        ('l', 'Low'),
        ('m', 'Medium'),
        ('h', 'High'),
        ], 'Income', sort=False, states = STATES)


    fam_apgar_help = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
        ], 'Help from family',
        help="Is the patient satisfied with the level of help coming from " \
        "the family when there is a problem ?", sort=False,
            states = STATES)

    fam_apgar_discussion = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
        ], 'Problems discussion',
        help="Is the patient satisfied with the level talking over the " \
        "problems as family ?", sort=False, states = STATES)

    fam_apgar_decisions = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
        ], 'Decision making',
        help="Is the patient satisfied with the level of making important " \
        "decisions as a group ?", sort=False, states = STATES)

    fam_apgar_timesharing = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
        ], 'Time sharing',
        help="Is the patient satisfied with the level of time that they " \
        "spend together ?", sort=False, states = STATES)

    fam_apgar_affection = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Moderately'),
        ('2', 'Very much'),
        ], 'Family affection',
        help="Is the patient satisfied with the level of affection coming " \
        "from the family ?", sort=False, states = STATES)

    fam_apgar_score = fields.Integer('Score',
        help="Total Family APGAR \n" \
        "7 - 10 : Functional Family \n" \
        "4 - 6  : Some level of disfunction \n" \
        "0 - 3  : Severe disfunctional family \n", states = STATES)

    education = fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
        ], 'Education Level', help="Education Level", sort=False,
            states = STATES)

    notes = fields.Text('Notes', states = STATES)

    state = fields.Selection([
        (None, ''),
        ('in_progress', 'In progress'),
        ('done', 'Done'),
        ], 'State', readonly=True, sort=False)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={'invisible': Equal(Eval('state'), 'in_progress')},
        help="Health Professional that finished the patient evaluation")


    @fields.depends('fam_apgar_help', 'fam_apgar_timesharing',
        'fam_apgar_discussion', 'fam_apgar_decisions', 'fam_apgar_affection')
    def on_change_with_fam_apgar_score(self):
        fam_apgar_help = int(self.fam_apgar_help or '0')
        fam_apgar_timesharing = int(self.fam_apgar_timesharing or '0')
        fam_apgar_discussion = int(self.fam_apgar_discussion or '0')
        fam_apgar_decisions = int(self.fam_apgar_decisions or '0')
        fam_apgar_affection = int(self.fam_apgar_affection or '0')
        total = (fam_apgar_help + fam_apgar_timesharing +
            fam_apgar_discussion + fam_apgar_decisions +
            fam_apgar_affection)

        return total

    @staticmethod
    def default_assessment_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'in_progress'

    @staticmethod
    def default_health_professional():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        health_professional = HealthProf.get_health_professional()
        return health_professional

    # Show the gender and age upon entering the patient 
    # These two are function fields (don't exist at DB level)
    @fields.depends('patient')
    def on_change_patient(self):
        gender=None
        age=''
        self.gender = self.patient.gender
        self.computed_age = self.patient.age

        occupation=education=du=housing=None
        if (self.patient and self.patient.name.occupation):
            occupation = self.patient.name.occupation
        
        if (self.patient and self.patient.name.education):
            education = self.patient.name.education

        if (self.patient and self.patient.name.du):
            du = self.patient.name.du

        if (self.patient and self.patient.name.du):
            housing = self.patient.name.du.housing

        self.occupation = occupation
        self.education = education
        self.du = du
        self.housing = housing



    def get_patient_gender(self, name):
        return self.patient.gender

    @classmethod
    def search_patient_gender(cls, name, clause):
        res = []
        value = clause[2]
        res.append(('patient.name.gender', clause[1], value))
        return res

    @classmethod
    @ModelView.button
    def end_assessment(cls, assessments):
        
        assessment_id = assessments[0]

        # Change the state of the assessment to "Done"
        HealthProf= Pool().get('gnuhealth.healthprofessional')

        signing_hp = HealthProf.get_health_professional()

        cls.write(assessments, {
            'state': 'done',
            'signed_by': signing_hp,
            })
        

    def patient_age_at_assessment(self, name):
        if (self.patient.name.dob and self.assessment_date):
            rdelta = relativedelta (self.assessment_date.date(),
                self.patient.name.dob)
            years_months_days = str(rdelta.years) + 'y ' \
                + str(rdelta.months) + 'm ' \
                + str(rdelta.days) + 'd'
            return years_months_days
        else:
            return None


    @classmethod
    def __setup__(cls):
        super(PatientSESAssessment, cls).__setup__()

        cls._buttons.update({
            'end_assessment': {'invisible': Equal(Eval('state'), 'done')}
            })



class GnuHealthPatient(ModelSQL, ModelView):
    __name__ = 'gnuhealth.patient'


    occupation = fields.Function(fields.Many2One('gnuhealth.occupation','Occupation'), 'get_patient_occupation')

    education = fields.Function(fields.Selection([
        (None, ''),
        ('0', 'None'),
        ('1', 'Incomplete Primary School'),
        ('2', 'Primary School'),
        ('3', 'Incomplete Secondary School'),
        ('4', 'Secondary School'),
        ('5', 'University'),
        ], 'Education Level', help="Education Level", sort=False), 'get_patient_education')


    housing = fields.Function(fields.Selection([
        (None, ''),
        ('0', 'Shanty, deficient sanitary conditions'),
        ('1', 'Small, crowded but with good sanitary conditions'),
        ('2', 'Comfortable and good sanitary conditions'),
        ('3', 'Roomy and excellent sanitary conditions'),
        ('4', 'Luxury and excellent sanitary conditions'),
        ], 'Housing conditions', help="Housing and sanitary living conditions", sort=False), 'get_patient_housing')


    ses_assessments = fields.One2Many('gnuhealth.ses.assessment',
        'patient','Assessments',readonly=True,
        help="Socioeconomics and Family assessments history")


    hostile_area = fields.Boolean('Hostile Area',
        help="Check if patient lives in a zone of high hostility (eg, war)")

    single_parent = fields.Boolean('Single parent family')
    domestic_violence = fields.Boolean('Domestic violence')
    working_children = fields.Boolean('Working children')
    teenage_pregnancy = fields.Boolean('Teenage pregnancy')
    sexual_abuse = fields.Boolean('Sexual abuse')
    drug_addiction = fields.Boolean('Drug addiction')
    school_withdrawal = fields.Boolean('School withdrawal')
    prison_past = fields.Boolean('Has been in prison')
    prison_current = fields.Boolean('Currently in prison')
    relative_in_prison = fields.Boolean('Relative in prison',
        help="Check if someone from the nuclear family - parents / " \
        "sibblings  is or has been in prison")

    ses_notes = fields.Text('Extra info')


    # GnuHealth 2.0 . Occupation and Education are now functional fields.
    # Retrives the information from the party model.
    occupation = fields.Function(fields.Many2One('gnuhealth.occupation',
        'Occupation'), 'get_patient_occupation')


    works_at_home = fields.Boolean('Works at home',
        help="Check if the patient works at his / her house")
    hours_outside = fields.Integer('Hours outside home',
        help="Number of hours a day the patient spend outside the house")


    def get_patient_occupation(self, name):
        if (self.name.occupation):
            return self.name.occupation.id

    def get_patient_education(self, name):
        return self.name.education

    def get_patient_housing(self, name):
        if (self.name.du):
            return self.name.du.housing



from .base_classes import *
from .support_functions import *


class ImmunizationRecommendation(Resource):
    """A patient's point-of-time immunization status and recommendation
    with optional supporting justification.If the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, subject=None, recommendation=None):
        self.original_tagname_ = None
        super(ImmunizationRecommendation, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.subject = subject
        if recommendation is None:
            self.recommendation = []
        else:
            self.recommendation = recommendation
    def factory(*args_, **kwargs_):
        if ImmunizationRecommendation.subclass:
            return ImmunizationRecommendation.subclass(*args_, **kwargs_)
        else:
            return ImmunizationRecommendation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_recommendation(self): return self.recommendation
    def set_recommendation(self, recommendation): self.recommendation = recommendation
    def add_recommendation(self, value): self.recommendation.append(value)
    def insert_recommendation(self, index, value): self.recommendation[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.subject is not None or
            self.recommendation or
            super(ImmunizationRecommendation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImmunizationRecommendation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImmunizationRecommendation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImmunizationRecommendation'):
        super(ImmunizationRecommendation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation')
    def exportChildren(self, outfile, level, namespace_='', name_='ImmunizationRecommendation', fromsubclass_=False, pretty_print=True):
        super(ImmunizationRecommendation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        for recommendation_ in self.recommendation:
            recommendation_.export(outfile, level, namespace_, name_='recommendation', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImmunizationRecommendation', mapping_=None):
        element = super(ImmunizationRecommendation, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        for recommendation_ in self.recommendation:
            recommendation_.to_etree(element, name_='recommendation', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImmunizationRecommendation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'recommendation':
            obj_ = ImmunizationRecommendation_Recommendation.factory()
            obj_.build(child_)
            self.recommendation.append(obj_)
            obj_.original_tagname_ = 'recommendation'
        super(ImmunizationRecommendation, self).buildChildren(child_, node, nodeName_, True)
# end class ImmunizationRecommendation


class ImmunizationRecommendation_Recommendation(BackboneElement):
    """A patient's point-of-time immunization status and recommendation
    with optional supporting justification."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, date=None, vaccineType=None, doseNumber=None, forecastStatus=None, dateCriterion=None, protocol=None, supportingImmunization=None, supportingPatientInformation=None):
        self.original_tagname_ = None
        super(ImmunizationRecommendation_Recommendation, self).__init__(id, extension, modifierExtension, )
        self.date = date
        self.vaccineType = vaccineType
        self.doseNumber = doseNumber
        self.forecastStatus = forecastStatus
        if dateCriterion is None:
            self.dateCriterion = []
        else:
            self.dateCriterion = dateCriterion
        self.protocol = protocol
        if supportingImmunization is None:
            self.supportingImmunization = []
        else:
            self.supportingImmunization = supportingImmunization
        if supportingPatientInformation is None:
            self.supportingPatientInformation = []
        else:
            self.supportingPatientInformation = supportingPatientInformation
    def factory(*args_, **kwargs_):
        if ImmunizationRecommendation_Recommendation.subclass:
            return ImmunizationRecommendation_Recommendation.subclass(*args_, **kwargs_)
        else:
            return ImmunizationRecommendation_Recommendation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_vaccineType(self): return self.vaccineType
    def set_vaccineType(self, vaccineType): self.vaccineType = vaccineType
    def get_doseNumber(self): return self.doseNumber
    def set_doseNumber(self, doseNumber): self.doseNumber = doseNumber
    def get_forecastStatus(self): return self.forecastStatus
    def set_forecastStatus(self, forecastStatus): self.forecastStatus = forecastStatus
    def get_dateCriterion(self): return self.dateCriterion
    def set_dateCriterion(self, dateCriterion): self.dateCriterion = dateCriterion
    def add_dateCriterion(self, value): self.dateCriterion.append(value)
    def insert_dateCriterion(self, index, value): self.dateCriterion[index] = value
    def get_protocol(self): return self.protocol
    def set_protocol(self, protocol): self.protocol = protocol
    def get_supportingImmunization(self): return self.supportingImmunization
    def set_supportingImmunization(self, supportingImmunization): self.supportingImmunization = supportingImmunization
    def add_supportingImmunization(self, value): self.supportingImmunization.append(value)
    def insert_supportingImmunization(self, index, value): self.supportingImmunization[index] = value
    def get_supportingPatientInformation(self): return self.supportingPatientInformation
    def set_supportingPatientInformation(self, supportingPatientInformation): self.supportingPatientInformation = supportingPatientInformation
    def add_supportingPatientInformation(self, value): self.supportingPatientInformation.append(value)
    def insert_supportingPatientInformation(self, index, value): self.supportingPatientInformation[index] = value
    def hasContent_(self):
        if (
            self.date is not None or
            self.vaccineType is not None or
            self.doseNumber is not None or
            self.forecastStatus is not None or
            self.dateCriterion or
            self.protocol is not None or
            self.supportingImmunization or
            self.supportingPatientInformation or
            super(ImmunizationRecommendation_Recommendation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.Recommendation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.Recommendation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImmunizationRecommendation.Recommendation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImmunizationRecommendation.Recommendation'):
        super(ImmunizationRecommendation_Recommendation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.Recommendation')
    def exportChildren(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.Recommendation', fromsubclass_=False, pretty_print=True):
        super(ImmunizationRecommendation_Recommendation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.vaccineType is not None:
            self.vaccineType.export(outfile, level, namespace_, name_='vaccineType', pretty_print=pretty_print)
        if self.doseNumber is not None:
            self.doseNumber.export(outfile, level, namespace_, name_='doseNumber', pretty_print=pretty_print)
        if self.forecastStatus is not None:
            self.forecastStatus.export(outfile, level, namespace_, name_='forecastStatus', pretty_print=pretty_print)
        for dateCriterion_ in self.dateCriterion:
            dateCriterion_.export(outfile, level, namespace_, name_='dateCriterion', pretty_print=pretty_print)
        if self.protocol is not None:
            self.protocol.export(outfile, level, namespace_, name_='protocol', pretty_print=pretty_print)
        for supportingImmunization_ in self.supportingImmunization:
            supportingImmunization_.export(outfile, level, namespace_, name_='supportingImmunization', pretty_print=pretty_print)
        for supportingPatientInformation_ in self.supportingPatientInformation:
            supportingPatientInformation_.export(outfile, level, namespace_, name_='supportingPatientInformation', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImmunizationRecommendation.Recommendation', mapping_=None):
        element = super(ImmunizationRecommendation_Recommendation, self).to_etree(parent_element, name_, mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.vaccineType is not None:
            vaccineType_ = self.vaccineType
            vaccineType_.to_etree(element, name_='vaccineType', mapping_=mapping_)
        if self.doseNumber is not None:
            doseNumber_ = self.doseNumber
            doseNumber_.to_etree(element, name_='doseNumber', mapping_=mapping_)
        if self.forecastStatus is not None:
            forecastStatus_ = self.forecastStatus
            forecastStatus_.to_etree(element, name_='forecastStatus', mapping_=mapping_)
        for dateCriterion_ in self.dateCriterion:
            dateCriterion_.to_etree(element, name_='dateCriterion', mapping_=mapping_)
        if self.protocol is not None:
            protocol_ = self.protocol
            protocol_.to_etree(element, name_='protocol', mapping_=mapping_)
        for supportingImmunization_ in self.supportingImmunization:
            supportingImmunization_.to_etree(element, name_='supportingImmunization', mapping_=mapping_)
        for supportingPatientInformation_ in self.supportingPatientInformation:
            supportingPatientInformation_.to_etree(element, name_='supportingPatientInformation', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImmunizationRecommendation_Recommendation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'vaccineType':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.vaccineType = obj_
            obj_.original_tagname_ = 'vaccineType'
        elif nodeName_ == 'doseNumber':
            obj_ = integer.factory()
            obj_.build(child_)
            self.doseNumber = obj_
            obj_.original_tagname_ = 'doseNumber'
        elif nodeName_ == 'forecastStatus':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.forecastStatus = obj_
            obj_.original_tagname_ = 'forecastStatus'
        elif nodeName_ == 'dateCriterion':
            obj_ = ImmunizationRecommendation_DateCriterion.factory()
            obj_.build(child_)
            self.dateCriterion.append(obj_)
            obj_.original_tagname_ = 'dateCriterion'
        elif nodeName_ == 'protocol':
            obj_ = ImmunizationRecommendation_Protocol.factory()
            obj_.build(child_)
            self.protocol = obj_
            obj_.original_tagname_ = 'protocol'
        elif nodeName_ == 'supportingImmunization':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.supportingImmunization.append(obj_)
            obj_.original_tagname_ = 'supportingImmunization'
        elif nodeName_ == 'supportingPatientInformation':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.supportingPatientInformation.append(obj_)
            obj_.original_tagname_ = 'supportingPatientInformation'
        super(ImmunizationRecommendation_Recommendation, self).buildChildren(child_, node, nodeName_, True)
# end class ImmunizationRecommendation_Recommendation


class ImmunizationRecommendation_DateCriterion(BackboneElement):
    """A patient's point-of-time immunization status and recommendation
    with optional supporting justification."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, value=None):
        self.original_tagname_ = None
        super(ImmunizationRecommendation_DateCriterion, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.value = value
    def factory(*args_, **kwargs_):
        if ImmunizationRecommendation_DateCriterion.subclass:
            return ImmunizationRecommendation_DateCriterion.subclass(*args_, **kwargs_)
        else:
            return ImmunizationRecommendation_DateCriterion(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            self.code is not None or
            self.value is not None or
            super(ImmunizationRecommendation_DateCriterion, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.DateCriterion', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.DateCriterion')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImmunizationRecommendation.DateCriterion', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImmunizationRecommendation.DateCriterion'):
        super(ImmunizationRecommendation_DateCriterion, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.DateCriterion')
    def exportChildren(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.DateCriterion', fromsubclass_=False, pretty_print=True):
        super(ImmunizationRecommendation_DateCriterion, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.value is not None:
            self.value.export(outfile, level, namespace_, name_='value', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImmunizationRecommendation.DateCriterion', mapping_=None):
        element = super(ImmunizationRecommendation_DateCriterion, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.value is not None:
            value_ = self.value
            value_.to_etree(element, name_='value', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImmunizationRecommendation_DateCriterion, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'value':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.value = obj_
            obj_.original_tagname_ = 'value'
        super(ImmunizationRecommendation_DateCriterion, self).buildChildren(child_, node, nodeName_, True)
# end class ImmunizationRecommendation_DateCriterion


class ImmunizationRecommendation_Protocol(BackboneElement):
    """A patient's point-of-time immunization status and recommendation
    with optional supporting justification."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, doseSequence=None, description=None, authority=None, series=None):
        self.original_tagname_ = None
        super(ImmunizationRecommendation_Protocol, self).__init__(id, extension, modifierExtension, )
        self.doseSequence = doseSequence
        self.description = description
        self.authority = authority
        self.series = series
    def factory(*args_, **kwargs_):
        if ImmunizationRecommendation_Protocol.subclass:
            return ImmunizationRecommendation_Protocol.subclass(*args_, **kwargs_)
        else:
            return ImmunizationRecommendation_Protocol(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_doseSequence(self): return self.doseSequence
    def set_doseSequence(self, doseSequence): self.doseSequence = doseSequence
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_authority(self): return self.authority
    def set_authority(self, authority): self.authority = authority
    def get_series(self): return self.series
    def set_series(self, series): self.series = series
    def hasContent_(self):
        if (
            self.doseSequence is not None or
            self.description is not None or
            self.authority is not None or
            self.series is not None or
            super(ImmunizationRecommendation_Protocol, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.Protocol', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.Protocol')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ImmunizationRecommendation.Protocol', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ImmunizationRecommendation.Protocol'):
        super(ImmunizationRecommendation_Protocol, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ImmunizationRecommendation.Protocol')
    def exportChildren(self, outfile, level, namespace_='', name_='ImmunizationRecommendation.Protocol', fromsubclass_=False, pretty_print=True):
        super(ImmunizationRecommendation_Protocol, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.doseSequence is not None:
            self.doseSequence.export(outfile, level, namespace_, name_='doseSequence', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.authority is not None:
            self.authority.export(outfile, level, namespace_, name_='authority', pretty_print=pretty_print)
        if self.series is not None:
            self.series.export(outfile, level, namespace_, name_='series', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ImmunizationRecommendation.Protocol', mapping_=None):
        element = super(ImmunizationRecommendation_Protocol, self).to_etree(parent_element, name_, mapping_)
        if self.doseSequence is not None:
            doseSequence_ = self.doseSequence
            doseSequence_.to_etree(element, name_='doseSequence', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.authority is not None:
            authority_ = self.authority
            authority_.to_etree(element, name_='authority', mapping_=mapping_)
        if self.series is not None:
            series_ = self.series
            series_.to_etree(element, name_='series', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(ImmunizationRecommendation_Protocol, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'doseSequence':
            obj_ = integer.factory()
            obj_.build(child_)
            self.doseSequence = obj_
            obj_.original_tagname_ = 'doseSequence'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'authority':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authority = obj_
            obj_.original_tagname_ = 'authority'
        elif nodeName_ == 'series':
            obj_ = string.factory()
            obj_.build(child_)
            self.series = obj_
            obj_.original_tagname_ = 'series'
        super(ImmunizationRecommendation_Protocol, self).buildChildren(child_, node, nodeName_, True)
# end class ImmunizationRecommendation_Protocol
